﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Encodings.ParameterVectorEncoding;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Programmable {
  [Item("ParameterVectorCreater", "Creates a parameter vector as received from a SimOptRunner.")]
  [StorableClass]
  public class ParameterVectorCreater : SingleSuccessorOperator, IParameterVectorCreator, IStochasticOperator {

    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }

    public ILookupParameter<ProgrammableProblemScript> ScriptParameter {
      get { return (ILookupParameter<ProgrammableProblemScript>)Parameters["Script"]; }
    }

    public ILookupParameter<ParameterVector> ParameterVectorParameter {
      get { return (ILookupParameter<ParameterVector>)Parameters["ParameterVector"]; }
    }

    #region Interface Implementations of Encoding-specific Creators
    ILookupParameter<BinaryVector> IBinaryVectorCreator.BinaryVectorParameter {
      get { return (ILookupParameter<BinaryVector>)Parameters["BinaryVector"]; }
    }
    IValueLookupParameter<IntValue> IBinaryVectorCreator.LengthParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["BinaryVectorLength"]; }
    }

    ILookupParameter<IntegerVector> IIntegerVectorCreator.IntegerVectorParameter {
      get { return (ILookupParameter<IntegerVector>)Parameters["IntegerVector"]; }
    }

    IValueLookupParameter<IntMatrix> IBoundedIntegerVectorOperator.BoundsParameter {
      get { return (IValueLookupParameter<IntMatrix>)Parameters["IntegerVectorBounds"]; }
    }

    IValueLookupParameter<IntValue> IIntegerVectorCreator.LengthParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["IntegerVectorLength"]; }
    }

    ILookupParameter<RealVector> IRealVectorCreator.RealVectorParameter {
      get { return (ILookupParameter<RealVector>)Parameters["RealVector"]; }
    }

    IValueLookupParameter<DoubleMatrix> IRealVectorCreator.BoundsParameter {
      get { return (IValueLookupParameter<DoubleMatrix>)Parameters["RealVectorBounds"]; }
    }

    IValueLookupParameter<IntValue> IRealVectorCreator.LengthParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["RealVectorLength"]; }
    }

    ILookupParameter<Permutation> IPermutationCreator.PermutationParameter {
      get { return (ILookupParameter<Permutation>)Parameters["Permutation"]; }
    }

    IValueParameter<PermutationType> IPermutationCreator.PermutationTypeParameter {
      get { return (IValueParameter<PermutationType>)Parameters["PermutationType"]; }
    }

    IValueLookupParameter<IntValue> IPermutationCreator.LengthParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["PermutationLength"]; }
    }
    #endregion

    [StorableConstructor]
    protected ParameterVectorCreater(bool deserializing) : base(deserializing) { }
    protected ParameterVectorCreater(ParameterVectorCreater original, Cloner cloner) : base(original, cloner) { }
    public ParameterVectorCreater() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The random number generator to use."));
      Parameters.Add(new LookupParameter<ProgrammableProblemScript>("Script", "The script that will execute the evaluation function and define the parameter vector."));
      Parameters.Add(new LookupParameter<ParameterVector>("ParameterVector", "The vector that holds the parameters."));
      Parameters.Add(new LookupParameter<BinaryVector>("BinaryVector", "The binary vector that links into the parameter vector."));
      Parameters.Add(new ValueLookupParameter<IntValue>("BinaryVectorLength", "The length of the binary vector."));
      Parameters.Add(new LookupParameter<IntegerVector>("IntegerVector", "The integer vector that links into the parameter vector."));
      Parameters.Add(new ValueLookupParameter<IntValue>("IntegerVectorLength", "The length of the integer vector."));
      Parameters.Add(new ValueLookupParameter<IntMatrix>("IntegerVectorBounds", "The bounds of the integer vector."));
      Parameters.Add(new LookupParameter<RealVector>("RealVector", "The real vector that links into the parameter vector."));
      Parameters.Add(new ValueLookupParameter<IntValue>("RealVectorLength", "The length of the real vector."));
      Parameters.Add(new ValueLookupParameter<DoubleMatrix>("RealVectorBounds", "The bounds of the real vector."));
      Parameters.Add(new LookupParameter<Permutation>("Permutation", "The permutation that links into the parameter vector."));
      Parameters.Add(new ValueParameter<PermutationType>("PermutationType", "The type of the permutation."));
      Parameters.Add(new ValueLookupParameter<IntValue>("PermutationLength", "The length of the permutation."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ParameterVectorCreater(this, cloner);
    }

    public override IOperation Apply() {
      var random = RandomParameter.ActualValue;
      var runner = ScriptParameter.ActualValue;
      if (runner.Instance == null) throw new InvalidOperationException("Script instance is null, maybe the code doesn't compile.");

      var vector = runner.Instance.GetParametersToOptimize();
      ParameterVectorParameter.ActualValue = vector;

      if (vector.BooleanParameters != null) {
        vector.BooleanParameters.Randomize(random);
        ((IBinaryVectorCreator)this).BinaryVectorParameter.ActualValue = vector.BooleanParameters;
      }
      if (vector.IntegerParameters != null) {
        vector.IntegerParameters.Randomize(random, vector.IntegerBounds);
        ((IIntegerVectorCreator)this).IntegerVectorParameter.ActualValue = vector.IntegerParameters;
      }
      if (vector.RealParameters != null) {
        vector.RealParameters.Randomize(random, vector.RealBounds);
        ((IRealVectorCreator)this).RealVectorParameter.ActualValue = vector.RealParameters;
      }
      if (vector.PermutationParameter != null) {
        vector.PermutationParameter.Randomize(random);
        ((IPermutationCreator)this).PermutationParameter.ActualValue = vector.PermutationParameter;
      }
      return base.Apply();
    }
  }
}
